/*
 * (C) Copyright 2004-2011
 * Texas Instruments, <www.ti.com>
 *
 * Author :
 *	Maxim Podbereznyy <support@mentorel.com>
 *
 * Derived from Beagle Board and 3430 SDP code by
 *	Richard Woodruff <r-woodruff2@ti.com>
 *	Syed Mohammed Khasim <khasim@ti.com>
 *
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */
#include <common.h>
#ifdef CONFIG_STATUS_LED
#include <status_led.h>
#endif
#include <twl4030.h>
#include <linux/mtd/nand.h>
#include <asm/io.h>
#include <asm/arch/mmc_host_def.h>
#include <asm/arch/mux.h>
#include <asm/arch/mem.h>
#include <asm/arch/sys_proto.h>
#include <asm/gpio.h>
#include <asm/mach-types.h>
#include "somiq_am37.h"
#include <command.h>

#define pr_debug(fmt, args...) debug(fmt, ##args)

#define TWL4030_I2C_BUS			0
#define EXPANSION_EEPROM_I2C_BUS	1
#define EXPANSION_EEPROM_I2C_ADDRESS	0x50
#define CONFIG_SYS_I2C_EEPROM_ADDR      0x50    /* EEPROM 24AA02E48 */
#define CONFIG_SYS_I2C_EEPROM_OFF_ADDR  0xFA    /* EEPROM 24AA02E48 */
#define CONFIG_SYS_I2C_EEPROM_LEN       6       /* Bytes of address */

static int read_eeprom(void);

DECLARE_GLOBAL_DATA_PTR;

/*
 * Routine: board_init
 * Description: Early hardware init.
 */
int board_init(void)
{
	gpmc_init(); /* in SRAM or SDRAM, finish GPMC */
	/* board id for Linux */
	gd->bd->bi_arch_number = MACH_TYPE_SOMIQ_AM37;
	/* boot param addr */
	gd->bd->bi_boot_params = (OMAP34XX_SDRC_CS0 + 0x100);

#if defined(CONFIG_STATUS_LED) && defined(STATUS_LED_BOOT)
	status_led_set (STATUS_LED_BOOT, STATUS_LED_ON);
#endif

	return 0;
}

#ifdef CONFIG_SPL_BUILD

/* Macro to construct MCFG for DDR memory */
#define MCFG2(ramsize, raswidth, caswidth)					\
		V_MCFG_RASWIDTH(raswidth) | V_MCFG_CASWIDTH(caswidth) |	\
		V_MCFG_ADDRMUXLEGACY_FLEX | V_MCFG_RAMSIZE(ramsize) |	\
		V_MCFG_BANKALLOCATION_RBC | V_MCFG_B32NOT16_32 |	\
		V_MCFG_DEEPPD_EN | V_MCFG_RAMTYPE_DDR
/*
 * Routine: get_board_mem_timings
 * Description: If we use SPL then there is no x-loader nor config header
 * so we have to setup the DDR timings ourself on both banks.
 */
void get_board_mem_timings(u32 *mcfg, u32 *ctrla, u32 *ctrlb, u32 *rfr_ctrl,
		u32 *mr)
{
	int pop_mfr, pop_id;

	identify_nand_chip(&pop_mfr, &pop_id);

	*mr = MICRON_V_MR_165;
#if defined(CONFIG_DDRSIZE_256M)
	/* 256MB DDR */
	*mcfg = MCFG2((256 << 20), 14, 10);
	*ctrla = MICRON_V_ACTIMA_165;
	*ctrlb = MICRON_V_ACTIMB_165;
	*rfr_ctrl = SDP_3430_SDRC_RFR_CTRL_165MHz;
#elif defined(CONFIG_DDRSIZE_512M)
	/* 2 chips 256M x16: 512MB DDR */
	*mcfg = MCFG2((512 << 20), 14, 11);
	*ctrla = MICRON_V_ACTIMA_165;
	*ctrlb = MICRON_V_ACTIMB_165;
	*rfr_ctrl = SDP_3430_SDRC_RFR_CTRL_165MHz;
#else
	/* Assume 128MB and Micron/165MHz timings to be safe */
	*mcfg = MICRON_V_MCFG_165(128 << 20);
	*ctrla = MICRON_V_ACTIMA_165;
	*ctrlb = MICRON_V_ACTIMB_165;
	*rfr_ctrl = SDP_3430_SDRC_RFR_CTRL_165MHz;
#endif
}
#endif

/*
 * Configure DSS to display background color on DVID
 * Configure VENC to display color bar on S-Video
 */
void somiq_display_init(void)
{
	omap3_dss_venc_config(&venc_config_std_tv, VENC_HEIGHT, VENC_WIDTH);
	omap3_dss_panel_config(&dvid_cfg_xm);
}

/*
 * Enable DVI power
 */
static void somiq_dvi_pup(void)
{
	gpio_request(10, "");
	gpio_direction_output(10, 0);
	gpio_set_value(10, 1);
}

/*
 * Routine: misc_init_r
 * Description: Configure board specific parts
 */
int misc_init_r(void)
{
	struct control_prog_io *prog_io_base = (struct control_prog_io *)OMAP34XX_CTRL_BASE;

	/* Enable i2c2 pullup resisters */
	writel(~(PRG_I2C2_PULLUPRESX), &prog_io_base->io1);

	twl4030_power_init();
	twl4030_led_init(TWL4030_LED_LEDEN_LEDAON | TWL4030_LED_LEDEN_LEDBON);

	dieid_num_r();

	somiq_dvi_pup();
	somiq_display_init();
	omap3_dss_enable();

	/* Read EEPROM with EUI-48 */
	read_eeprom();

	return 0;
}

/*
 * Routine: set_muxconf_regs
 * Description: Setting up the configuration Mux registers specific to the
 *		hardware. Many pins need to be moved from protect to primary
 *		mode.
 */
void set_muxconf_regs(void)
{
	MUX_SOMIQ_AM37();
	MUX_BBTOYS_WIFI();
}

#if defined(CONFIG_GENERIC_MMC) && !defined(CONFIG_SPL_BUILD)
int board_mmc_init(bd_t *bis)
{
	omap_mmc_init(0);
	return 0;
}
#endif

/**
 * read_eeprom - read the EEPROM into memory
 */
static int read_eeprom(void)
{
	int ret;
	char e[CONFIG_SYS_I2C_EEPROM_LEN];
	char str[sizeof("00:00:00:00:00:00")];

	i2c_set_bus_num(EXPANSION_EEPROM_I2C_BUS);

	/* Check if expansion EEPROM is there*/
	if(i2c_probe(CONFIG_SYS_I2C_EEPROM_ADDR) == 0) {
		/* EEPROM found, read configuration data */
		ret = i2c_read(CONFIG_SYS_I2C_EEPROM_ADDR,
				CONFIG_SYS_I2C_EEPROM_OFF_ADDR,
				1, (uchar *)&e, sizeof(e));
		if (!ret) {
			/* Set Node address */
			sprintf(str,"%02x:%02x:%02x:%02x:%02x:%02x",
					e[0],e[1],e[2],e[3],e[4],e[5]);
			setenv("ethmac", str);
			printf("MAC address EUI-48 is %s\n",str);
		}
		else printf("EEPROM read failed\n");
	}
	else printf("EEPROM probe failed\n");

	/* Switch back to default bus and pin mux */
	i2c_set_bus_num(TWL4030_I2C_BUS);

	return ret;
}
